@@ -14,7 +14,11 @@ import os
 import time
 import uuid
 
+from cinderclient.v2 import client as cinderclient
 import fixtures
+from keystoneauth1 import loading
+from keystoneauth1 import session as ksession
+from keystoneclient import client as keystoneclient
 import os_client_config
 import six
 import tempest_lib.cli.base
@@ -23,6 +27,10 @@ import testtools
 import novaclient
 import novaclient.api_versions
 import novaclient.client
+import novaclient.v2.shell
+
+BOOT_IS_COMPLETE = ("login as 'cirros' user. default password: "
+                    "'cubswin:)'. use 'sudo' for root.")
 
 
 # The following are simple filter functions that filter our available
@@ -45,6 +53,16 @@ def pick_image(images):
     raise NoImageException()
 
 
+def pick_network(networks):
+    network_name = os.environ.get('OS_NOVACLIENT_NETWORK')
+    if network_name:
+        for network in networks:
+            if network.label == network_name:
+                return network
+        raise NoNetworkException()
+    return networks[0]
+
+
 class NoImageException(Exception):
     """We couldn't find an acceptable image."""
     pass
@@ -55,6 +73,11 @@ class NoFlavorException(Exception):
     pass
 
 
+class NoNetworkException(Exception):
+    """We couldn't find an acceptable network."""
+    pass
+
+
 class NoCloudConfigException(Exception):
     """We couldn't find a cloud configuration."""
     pass
@@ -147,18 +170,30 @@ class ClientTestBase(testtools.TestCase):
         passwd = auth_info['password']
         tenant = auth_info['project_name']
         auth_url = auth_info['auth_url']
+        self.project_domain_id = auth_info['project_domain_id']
+        if 'insecure' in cloud_config.config:
+            self.insecure = cloud_config.config['insecure']
+        else:
+            self.insecure = False
 
         if self.COMPUTE_API_VERSION == "2.latest":
             version = novaclient.API_MAX_VERSION.get_string()
         else:
             version = self.COMPUTE_API_VERSION or "2"
-        self.client = novaclient.client.Client(
-            version, user, passwd, tenant,
-            auth_url=auth_url)
+
+        loader = loading.get_plugin_loader("password")
+        auth = loader.load_from_options(username=user,
+                                        password=passwd,
+                                        project_name=tenant,
+                                        auth_url=auth_url)
+        session = ksession.Session(auth=auth, verify=(not self.insecure))
+
+        self.client = novaclient.client.Client(version, session=session)
 
         # pick some reasonable flavor / image combo
         self.flavor = pick_flavor(self.client.flavors.list())
         self.image = pick_image(self.client.images.list())
+        self.network = pick_network(self.client.networks.list())
 
         # create a CLI client in case we'd like to do CLI
         # testing. tempest_lib does this really weird thing where it
@@ -174,7 +209,13 @@ class ClientTestBase(testtools.TestCase):
             password=passwd,
             tenant_name=tenant,
             uri=auth_url,
-            cli_dir=cli_dir)
+            cli_dir=cli_dir,
+            insecure=self.insecure)
+
+        self.keystone = keystoneclient.Client(session=session,
+                                              username=user,
+                                              password=passwd)
+        self.cinder = cinderclient.Client(auth=auth, session=session)
 
     def nova(self, action, flags='', params='', fail_ok=False,
              endpoint_type='publicURL', merge_stderr=False):
@@ -187,14 +228,14 @@ class ClientTestBase(testtools.TestCase):
                                poll_interval=1):
         """Wait until volume reaches given status.
 
-        :param volume_id: uuid4 id of given volume
+        :param volume: volume resource
         :param status: expected status of volume
         :param timeout: timeout in seconds
         :param poll_interval: poll interval in seconds
         """
         start_time = time.time()
         while time.time() - start_time < timeout:
-            volume = self.client.volumes.get(volume.id)
+            volume = self.cinder.volumes.get(volume.id)
             if volume.status == status:
                 break
             time.sleep(poll_interval)
@@ -202,6 +243,23 @@ class ClientTestBase(testtools.TestCase):
             self.fail("Volume %s did not reach status %s after %d s"
                       % (volume.id, status, timeout))
 
+    def wait_for_server_os_boot(self, server_id, timeout=60,
+                                poll_interval=1):
+        """Wait until instance's operating system  is completely booted.
+
+        :param server_id: uuid4 id of given instance
+        :param timeout: timeout in seconds
+        :param poll_interval: poll interval in seconds
+        """
+        start_time = time.time()
+        while time.time() - start_time < timeout:
+            if BOOT_IS_COMPLETE in self.nova('console-log %s ' % server_id):
+                break
+            time.sleep(poll_interval)
+        else:
+            self.fail("Server %s did not boot after %d s"
+                      % (server_id, timeout))
+
     def wait_for_resource_delete(self, resource, manager,
                                  timeout=60, poll_interval=1):
         """Wait until getting the resource raises NotFound exception.
@@ -285,3 +343,77 @@ class ClientTestBase(testtools.TestCase):
                     return line.split("|")[1:-1][column_index].strip()
 
         raise ValueError("Unable to find value for column '%s'.")
+
+    def _create_server(self, name=None, with_network=True, add_cleanup=True,
+                       **kwargs):
+        name = name or self.name_generate(prefix='server')
+        if with_network:
+            nics = [{"net-id": self.network.id}]
+        else:
+            nics = None
+        server = self.client.servers.create(name, self.image, self.flavor,
+                                            nics=nics, **kwargs)
+        if add_cleanup:
+            self.addCleanup(server.delete)
+        novaclient.v2.shell._poll_for_status(
+            self.client.servers.get, server.id,
+            'building', ['active'])
+        return server
+
+    def _get_project_id(self, name):
+        """Obtain project id by project name."""
+        if self.keystone.version == "v3":
+            project = self.keystone.projects.find(name=name)
+        else:
+            project = self.keystone.tenants.find(name=name)
+        return project.id
+
+
+class TenantTestBase(ClientTestBase):
+    """Base test class for additional tenant and user creation which
+    could be required in various test scenarios
+    """
+
+    def setUp(self):
+        super(TenantTestBase, self).setUp()
+        user_name = self.name_generate('v' + self.COMPUTE_API_VERSION)
+        project_name = self.name_generate('v' + self.COMPUTE_API_VERSION)
+        password = 'password'
+
+        if self.keystone.version == "v3":
+            project = self.keystone.projects.create(project_name,
+                                                    self.project_domain_id)
+            self.project_id = project.id
+            self.addCleanup(self.keystone.projects.delete, self.project_id)
+
+            self.user_id = self.keystone.users.create(
+                name=user_name, password=password,
+                default_project=self.project_id).id
+
+            for role in self.keystone.roles.list():
+                if "member" in role.name.lower():
+                    self.keystone.roles.grant(role.id, user=self.user_id,
+                                              project=self.project_id)
+                    break
+        else:
+            project = self.keystone.tenants.create(project_name)
+            self.project_id = project.id
+            self.addCleanup(self.keystone.tenants.delete, self.project_id)
+
+            self.user_id = self.keystone.users.create(
+                user_name, password, tenant_id=self.project_id).id
+
+        self.addCleanup(self.keystone.users.delete, self.user_id)
+        self.cli_clients_2 = tempest_lib.cli.base.CLIClient(
+            username=user_name,
+            password=password,
+            tenant_name=project_name,
+            uri=self.cli_clients.uri,
+            cli_dir=self.cli_clients.cli_dir,
+            insecure=self.insecure)
+
+    def another_nova(self, action, flags='', params='', fail_ok=False,
+                     endpoint_type='publicURL', merge_stderr=False):
+        flags += " --os-compute-api-version %s " % self.COMPUTE_API_VERSION
+        return self.cli_clients_2.nova(action, flags, params, fail_ok,
+                                       endpoint_type, merge_stderr)
